class GuiActions:
    tab = "↔TAB"
    leave   = "Sair"
    edit = "Ver tarefa"
    show_time = "Tempo"
    filter = "Filtrar"
    principal = "Principal"
    github = "Web   "
    help = "Ajuda"
    download = "Baixar"
    activate = "Ativar"
    navigate = "←↓→"
    uncheck = "Desmarcar"
    search = "Busca"
    pallete = "Paleta"
    grade = "Auto Avaliação"
    # tester frame
    time_limit = "Limite de tempo"
    evaluate_tester = "Testar"
    execute_tester  = "Executar"
    calibrate = "Calibrar teclas"
    draft = "Rascunho"
    all = "Todos"

class GuiKeys:
    calibrate = "c"

    inbox = "1"
    all_tasks = "2"
    
    panel_graph = "3"
    panel_logs = "4"
    panel_skills = "5"
    panel_help = "6"

    show_duration = "T"

    palette = "p"
    images = "I"
    down_task = "b"
    activate = "\n"
    reload_game = "R"
    
    create_draft = "r"
    set_lang_drafts = "L"
    open_url = "u"
    key_quit = "q"
    edit= "v"
    colors = "C"
    borders = "B"
    search = "/"
    self_evaluate = "a"
    delete_folder = "D"
    diff = "d"
    unfold_patch = "V"
    execute = "e"
    evaluate = "t"
    toggle_main = "\t"
    quit   = "q"
    lock = "u"
    limite  = "l"
    expand = ">"
    collapse = "<"